"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-unused-vars */
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const ts = __importStar(require("typescript"));
const filePath = path.resolve('../cwi-base/src/index.ts');
//const filePath = path.resolve('../../_dojo/sendover/src/index.ts')
const sourceText = fs.readFileSync(filePath, { encoding: "utf-8" });
const { name: fileName } = path.parse(filePath);
const compOpts = {
    allowJs: false,
    target: ts.ScriptTarget.Latest
};
const progOpts = {
    rootNames: [filePath],
    options: compOpts
};
const program = ts.createProgram(progOpts);
const sourceFiles = program.getSourceFiles().filter(sf => sf.fileName.indexOf('node_modules') === -1);
const printOpts = {
    newLine: ts.NewLineKind.LineFeed,
    removeComments: true,
    omitTrailingSemicolon: true
};
const printHandlers = {};
const printer = ts.createPrinter(printOpts, printHandlers);
const firstHeadingLevel = 2;
function headingLevelMd(relativeLevel) {
    return '#'.repeat(relativeLevel + firstHeadingLevel - 1);
}
function getJsDoc(node) {
    const jsDoc = (node['jsDoc']) ? node['jsDoc'] : [];
    const tags = [];
    for (const doc of jsDoc) {
        for (const tag of (doc['tags'] ? doc['tags'] : [])) {
            tags.push(tag);
        }
    }
    return { jsDoc, tags };
}
class DocBase {
    constructor(name, node, sf, label, labelPlural) {
        this.name = name;
        this.node = node;
        this.sf = sf;
        this.label = label;
        this.labelPlural = labelPlural;
        this.text = node.getText(sf);
        this.fullText = node.getFullText(sf);
        this.isPrivate = false;
        const r = getJsDoc(node);
        this.jsDoc = r.jsDoc;
        this.jsDocTags = r.tags;
        this.isPrivate = r.tags.some(t => t.kind === ts.SyntaxKind.JSDocPrivateTag);
    }
    toMarkDownRefLink() {
        return `[${this.name}](#${this.label.toLowerCase()}-${this.name.toLowerCase()})`;
    }
    toMarkDown() {
        let md = `${headingLevelMd(3)} ${this.label}: ${this.name}\n\n`;
        if (this.jsDoc.some(d => d.kind === ts.SyntaxKind.JSDoc)) {
            md += `${headingLevelMd(4)} Description\n\n`;
            for (const d of this.jsDoc) {
                if (d['comment'])
                    md += `${d['comment']}\n\n`;
            }
        }
        md += '```ts\n';
        md += this.toMarkDownTs();
        md += '\n```\n\n';
        md += this.toMarkDownDetails();
        return md;
    }
    toMarkDownTs() {
        return printer.printNode(ts.EmitHint.Unspecified, this.node, this.sf);
    }
    toMarkDownDetails() {
        return '';
    }
}
class DocType extends DocBase {
    constructor(name, node, sf) { super(name, node, sf, 'Type', 'Types'); }
}
class DocInterface extends DocBase {
    constructor(name, node, sf) {
        super(name, node, sf, 'Interface', 'Interfaces');
        this.id = node;
    }
    toMarkDownDetails() {
        let md = '';
        let detailsStartAdded = false;
        const addDetailsStart = () => {
            if (!detailsStartAdded) {
                detailsStartAdded = true;
                md += `<details>\n\n<summary>${this.label} ${this.name} Member Details</summary>\n\n`;
            }
        };
        const addDetailsEnd = () => {
            if (detailsStartAdded) {
                md += `</details>\n\n`;
            }
        };
        for (const m of this.id.members) {
            const ps = m;
            const propName = ps.name.escapedText;
            const r = getJsDoc(m);
            if (r.jsDoc.length > 0) {
                addDetailsStart();
                md += `**${propName}**\n\n`;
                for (const d of r.jsDoc) {
                    if (d['comment']) {
                        md += `${d['comment']}\n\n`;
                    }
                }
            }
        }
        addDetailsEnd();
        return md;
    }
}
class DocFunction extends DocBase {
    constructor(name, node, sf) {
        super(name, node, sf, 'Function', 'Functions');
        this.fd = node;
    }
    toMarkDownTs() {
        let mdts = printer.printNode(ts.EmitHint.Unspecified, this.node, this.sf);
        if (this.fd.body && !this.jsDocTags.some(t => t.kind === ts.SyntaxKind.JSDocTag
            && t.tagName.escapedText === 'publicbody')) {
            const bodyts = printer.printNode(ts.EmitHint.Unspecified, this.fd.body, this.sf);
            mdts = mdts.slice(0, mdts.length - bodyts.length);
        }
        return mdts;
    }
    toMarkDownDetails() {
        let md = '';
        const returnsTags = this.jsDocTags.filter(t => t.kind === ts.SyntaxKind.JSDocReturnTag)
            .map(t => t)
            .filter(t => !!t.comment);
        if (returnsTags.length > 0) {
            md += `${headingLevelMd(4)} Returns\n\n`;
            for (const t of returnsTags) {
                const rt = t;
                md += `${rt.comment}\n\n`;
            }
        }
        return md;
    }
}
class DocClass extends DocBase {
    constructor(name, node, sf) {
        super(name, node, sf, 'Class', 'Classes');
        this.cd = node;
    }
    toMarkDownTs() {
        // class definition typescript including all members and function bodies...
        let mdts = printer.printNode(ts.EmitHint.Unspecified, this.node, this.sf);
        for (const ce of this.cd.members.filter(m => m['body'])) {
            const r = getJsDoc(ce);
            if (!r.tags.some(t => t.tagName.escapedText === 'publicbody')) {
                // Remove the body from documentation typescript
                let bodyts = printer.printNode(ts.EmitHint.Unspecified, ce['body'], this.sf);
                bodyts = bodyts.replace(/\n/g, '\n    ');
                let pos = mdts.indexOf(bodyts);
                if (pos === -1) {
                    // set and get accessor bodies sometimes are inlined in full generated class typescript
                    bodyts = bodyts.replace(/\n */g, ' ');
                    pos = mdts.indexOf(bodyts);
                }
                if (pos > -1)
                    mdts = mdts.slice(0, pos) + mdts.slice(pos + bodyts.length);
            }
        }
        return mdts;
    }
    toMarkDownDetails() {
        let md = '';
        let detailsStartAdded = false;
        const addDetailsStart = () => {
            if (!detailsStartAdded) {
                detailsStartAdded = true;
                md += `<details>\n\n<summary>${this.label} ${this.name} Member Details</summary>\n\n`;
            }
        };
        const addDetailsEnd = () => {
            if (detailsStartAdded) {
                md += `</details>\n\n`;
            }
        };
        for (const m of this.cd.members) {
            switch (m.kind) {
                case ts.SyntaxKind.Constructor:
                    {
                        const n = m;
                    }
                    break;
                case ts.SyntaxKind.GetAccessor:
                    {
                        const n = m;
                    }
                    break;
                case ts.SyntaxKind.SetAccessor:
                    {
                        const n = m;
                    }
                    break;
                case ts.SyntaxKind.MethodDeclaration:
                    {
                        const n = m;
                    }
                    break;
                default:
                    break;
            }
            /*
            const ps = m as ts.PropertySignature
            const propName = (ps.name as ts.Identifier).escapedText
            const r = getJsDoc(m)
            if (r.jsDoc.length > 0) {
                addDetailsStart()
                md += `**${propName}**\n\n`
                for (const d of r.jsDoc) {
                    if (d['comment']) {
                        md += `${d['comment']}\n\n`
                    }
                }
            }
            */
        }
        addDetailsEnd();
        return md;
    }
}
class DocVariable extends DocBase {
    constructor(name, node, sf) {
        super(name, node, sf, 'Variable', 'Variables');
    }
}
const docTypes = [];
const docInterfaces = [];
const docFunctions = [];
const docClasses = [];
const docVariables = [];
for (const sf of sourceFiles) {
    console.log(`SourceFile ${sf.fileName}`);
    ts.forEachChild(sf, node => {
        var _a, _b, _c, _d;
        let name = "", ignore = false;
        if (ts.isTypeAliasDeclaration(node)) {
            const modFlags = ts.getCombinedModifierFlags(node);
            if (!(modFlags & ts.ModifierFlags.Export))
                return;
            name = ((_a = node.name) === null || _a === void 0 ? void 0 : _a.text) || "";
            docTypes.push(new DocType(name, node, sf));
        }
        else if (ts.isInterfaceDeclaration(node)) {
            const modFlags = ts.getCombinedModifierFlags(node);
            if (!(modFlags & ts.ModifierFlags.Export))
                return;
            name = ((_b = node.name) === null || _b === void 0 ? void 0 : _b.text) || "";
            docInterfaces.push(new DocInterface(name, node, sf));
        }
        else if (ts.isFunctionDeclaration(node)) {
            const modFlags = ts.getCombinedModifierFlags(node);
            if (!(modFlags & ts.ModifierFlags.Export))
                return;
            name = ((_c = node.name) === null || _c === void 0 ? void 0 : _c.text) || "";
            docFunctions.push(new DocFunction(name, node, sf));
        }
        else if (ts.isClassDeclaration(node)) {
            const modFlags = ts.getCombinedModifierFlags(node);
            if (!(modFlags & ts.ModifierFlags.Export))
                return;
            name = ((_d = node.name) === null || _d === void 0 ? void 0 : _d.text) || "";
            docClasses.push(new DocClass(name, node, sf));
        }
        else if (ts.isVariableStatement(node)) {
            if (!node.modifiers || !node.modifiers.some(m => m.kind === ts.SyntaxKind.ExportKeyword))
                return;
            for (const v of node.declarationList.declarations) {
                name = v.name.getText(sf) || "";
                docVariables.push(new DocVariable(name, node, sf));
            }
        }
        else if (node.kind === ts.SyntaxKind.ImportDeclaration) {
            ignore = true;
        }
        else if (node.kind === ts.SyntaxKind.EndOfFileToken ||
            node.kind === ts.SyntaxKind.NewLineTrivia ||
            node.kind === ts.SyntaxKind.WhitespaceTrivia) {
            ignore = true;
        }
        else if (node.kind === ts.SyntaxKind.SingleLineCommentTrivia ||
            node.kind === ts.SyntaxKind.MultiLineCommentTrivia) {
            ignore = true;
        }
        else {
            name = 'unknown';
        }
        if (!ignore) {
            //console.log(`### ${kind} ${name} ###`)
            //const output = printer.printNode(ts.EmitHint.Unspecified, node, sf)
            //console.log(output, '\n')
        }
    });
}
let md = `${headingLevelMd(1)} API\n\n`;
let docs = [docInterfaces, docClasses, docTypes, docFunctions, docVariables];
// Remove anything marked with '@private' JSDoc tag.
for (let i = 0; i < docs.length; i++) {
    docs[i] = docs[i].filter(d => !d.isPrivate);
}
// Eliminate empty doc categories
docs = docs.filter(d => d.length > 0);
let links = `Links: [API](#api)`;
for (const doc of docs) {
    links += `, [${doc[0].labelPlural}](#${doc[0].labelPlural.toLowerCase()})`;
}
links += '\n\n';
md += links;
for (const doc of docs) {
    doc.sort((a, b) => a.name < b.name ? -1 : a.name === b.name ? 0 : 1);
    md += `${headingLevelMd(2)} ${doc[0].labelPlural}\n\n`;
    let cols = 1, rows = doc.length;
    while (cols < 4 && rows > 12) {
        cols++;
        rows = Math.ceil(doc.length / cols);
    }
    for (let col = 1; col <= cols; col++) {
        md += '| ';
    }
    md += '|\n';
    for (let col = 1; col <= cols; col++) {
        md += '| --- ';
    }
    md += '|\n';
    for (let row = 1; row <= rows; row++) {
        for (let col = 1; col <= cols; col++) {
            const i = (col - 1) * rows + row - 1;
            const d = i < doc.length ? doc[i].toMarkDownRefLink() : '';
            if (col === 1)
                md += '|';
            md += ` ${d} |`;
        }
        md += '\n';
    }
    md += `\n${links}---\n\n`;
    for (const d of doc) {
        md += d.toMarkDown();
        md += links;
        md += '---\n';
    }
}
try {
    fs.unlinkSync('./Documentation.md');
}
catch ( /* */_a) { /* */ }
fs.writeFileSync('./Documentation.md', md);
// eslint-disable-next-line no-debugger
debugger;
console.log('done');
//# sourceMappingURL=old%20code.js.map