#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ts2md = exports.Ts2Md = void 0;
/* eslint-disable @typescript-eslint/no-unused-vars */
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const mdMerge_1 = require("./mdMerge");
/**
 * Parse available JSDoc information on this node.
 */
function getJsDocInfo(node, name, parent) {
    const r = {
        isPrivate: false,
        comments: [],
        params: [],
        returns: [],
        throws: [],
        examples: [],
        tags: [],
        other: [],
        publicBody: false,
        privateInitializer: false,
        properties: {}
    };
    if (name !== 'constructor' && parent && parent.jsDoc.properties[name])
        // Forward @property tags on the parent to this node by name
        r.comments.push(parent.jsDoc.properties[name]);
    const jsDoc = (node['jsDoc']) ? node['jsDoc'] : [];
    for (const doc of jsDoc) {
        const comment = doc['comment'];
        if (comment && typeof comment === 'string')
            r.comments.push(comment);
        for (const t of (doc['tags'] ? doc['tags'] : [])) {
            const comment = typeof t['comment'] === 'string' ? t['comment'] : undefined;
            if (t.kind === typescript_1.default.SyntaxKind.JSDocParameterTag) {
                if (comment)
                    r.params.push(t);
            }
            else if (t.kind === typescript_1.default.SyntaxKind.JSDocReturnTag) {
                if (comment)
                    r.returns.push(t);
            }
            else if (t.kind === typescript_1.default.SyntaxKind.JSDocThrowsTag) {
                if (comment)
                    r.throws.push(t);
            }
            else {
                const tag = t.kind === typescript_1.default.SyntaxKind.JSDocTag ? t.tagName.escapedText : '';
                if (tag === 'publicbody') {
                    r.publicBody = true;
                }
                else if (tag === 'privateinitializer') {
                    r.privateInitializer = true;
                }
                else if (tag === 'property') {
                    if (comment) {
                        const re = /(?<identifier>[^ \t\-:]+)[ \t\-:]+(?<comment>.*)/;
                        const m = comment.match(re);
                        if (m && m.index === 0 && m.groups) {
                            r.properties[m.groups.identifier] = m.groups.comment;
                        }
                    }
                }
                else if (tag === 'example') {
                    if (comment) {
                        let example = comment;
                        if (example.indexOf('```') === -1)
                            example = '```ts\n' + example + '\n```\n';
                        r.examples.push(example);
                    }
                }
                else
                    r.tags.push(t);
            }
        }
    }
    return r;
}
/**
 * Wrapper for a Typescript `Node` of a specific derived type,
 * which is of interest for documentation generation.
 */
class DocItem {
    /**
     * This is really here just for demonstration / testing purposes...
     * @param item The typescript Node for this doc item.
     * @param name The name for this doc item.
     * @param sf The source file which defined this item.
     */
    constructor(item, name, sf, parent) {
        this.item = item;
        this.name = name;
        this.sf = sf;
        this.parent = parent;
        /**
         * Subsidiary documentation nodes when the node has members which
         * are themselves represented as documentation nodes.
         */
        this.memberDocs = [];
        this.jsDoc = getJsDocInfo(item, name, parent);
    }
}
class DocBase {
    constructor(sup, label, labelPlural, detailsLabel = 'Details') {
        this.sup = sup;
        this.label = label;
        this.labelPlural = labelPlural;
        this.detailsLabel = detailsLabel;
        this.docItems = [];
    }
    tryAddItem(s, sf, parent) {
        const items = this.filterItem(s);
        for (const item of items) {
            const docItem = new DocItem(item, this.getName(item, sf), sf, parent);
            if (!docItem.jsDoc.isPrivate || this.sup.nothingPrivate) {
                docItem.memberDocs = this.extractMemberDocs(docItem);
                for (const md of docItem.memberDocs) {
                    md.docItems.sort((a, b) => a.name < b.name ? -1 : a.name === b.name ? 0 : 1);
                }
                this.docItems.push(docItem);
            }
        }
    }
    extractMemberDocs(docItem) { return []; }
    isNotPrivate(item) {
        var _a;
        // not private if either we don't care if its private
        const notPrivate = (this.sup.nothingPrivate ||
            // Or it isn't private, which is either by having the private keyword
            !((item['modifiers'] && item['modifiers'].some(m => m.kind === typescript_1.default.SyntaxKind.PrivateKeyword)) ||
                (
                // or by having private jsdoc tag
                (_a = item['jsDoc']) === null || _a === void 0 ? void 0 : _a.some(t => typescript_1.default.isJSDocPrivateTag(t)))));
        return notPrivate;
    }
    findTs(findInTs, targetTs) {
        let pos = findInTs.indexOf(targetTs);
        if (pos === -1) {
            // Try outdenting once
            targetTs = targetTs.replace(/\n/g, '\n    ');
            pos = findInTs.indexOf(targetTs);
            if (pos === -1) {
                // set and get accessor bodies sometimes are inlined in full generated class typescript
                // remove all indenting
                targetTs = targetTs.replace(/\n */g, ' ');
                pos = findInTs.indexOf(targetTs);
            }
        }
        return { pos, len: targetTs.length };
    }
    removeTs(fromTs, removeTs, withSemi) {
        const r = this.findTs(fromTs, removeTs);
        // See if we would leave a dangling semicolon behind
        if (r.pos > -1 && withSemi && r.pos + r.len + 1 < fromTs.length && fromTs[r.pos + r.len] === ';' && fromTs[r.pos + r.len + 1] === '\n') {
            r.len += 2;
            // and remove leading spaces
            while (r.pos > 0 && fromTs[r.pos - 1] === ' ') {
                r.pos--;
                r.len++;
            }
        }
        if (r.pos > -1) {
            fromTs = fromTs.slice(0, r.pos) + fromTs.slice(r.pos + r.len);
            if (fromTs[r.pos] === '\n') {
                let pos2 = r.pos - 1;
                while (pos2 > 0 && fromTs[pos2] === ' ')
                    pos2--;
                if (fromTs[pos2] === '\n') {
                    // and remove blank line left after original removal
                    fromTs = fromTs.slice(0, pos2) + fromTs.slice(r.pos);
                }
            }
        }
        return fromTs;
    }
    /**
     * Base class implementation of markdown generation for a top level typescript AST node (`DocItem`).
     *
     * Adds relative level 3 heading with `label` and `docItem.name`
     *
     * Adds the nodes simple (no `@` tag) JSDoc nodes under relative level 4 'Description` heading
     *
     * Calls the `toMarkDownTs` override to add the typescript syntax code block for this node.
     *
     * Calls the `toMarkDownDtails` override to add any details markdown for this node.
     *
     * @returns the generated markdown for this `DocItem`
     */
    toMarkDown(docItem) {
        let md = `${this.sup.headingLevelMd(3)} ${this.label}: ${docItem.name}\n\n`;
        md += this.commentsDetails(docItem);
        md += this.examplesDetails(docItem);
        const mdts = this.toMarkDownTs(docItem);
        if (mdts) {
            md += '```ts\n' + mdts + '\n```\n\n';
        }
        const details = this.toMarkDownDetails(docItem);
        if (details) {
            md += `<details>\n\n<summary>${this.label} ${docItem.name} ${this.detailsLabel}</summary>\n\n`;
            md += details;
            md += `</details>\n\n`;
        }
        return md;
    }
    /**
     * Generate the typescript syntax for this node to be inserted in a typescript syntax code block
     * in generated markdown.
     *
     * Base class implementation uses the typescript compiler printer on `DocItem` AST node `item`.
     *
     * CAUTION: This adds ALL the source code for this item to the generated markdown. Override SHOULD
     * implement appropriate ommission control policies.
     *
     * @returns typescript syntax to be added within a typescript syntax code block for this `DocItem`
     */
    toMarkDownTs(docItem) {
        let item = docItem.item;
        if (item['initializer'] && docItem.jsDoc.privateInitializer) {
            item = Object.assign({}, item);
            item['initializer'] = undefined;
        }
        const mdts = this.sup.printer.printNode(typescript_1.default.EmitHint.Unspecified, item, docItem.sf);
        return mdts;
    }
    /**
     * Generate the 'Details' markdown (including ) for this node.
     *
     * Base class implementation returns an empty string.
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    toMarkDownDetails(docItem) {
        return '';
    }
    toMarkDownRefLink(docItem) {
        return `[${docItem.name}](#${this.label.toLowerCase()}-${docItem.name.toLowerCase()})`;
    }
    isExportedDeclaration(item) {
        const modFlags = typescript_1.default.getCombinedModifierFlags(item);
        const hasExportFlag = !!(modFlags & typescript_1.default.ModifierFlags.Export);
        return hasExportFlag;
    }
    argumentsDetails(docItem) {
        let md = '';
        if (docItem.jsDoc.params.length > 0) {
            md += `Argument Details\n\n`;
            for (const tag of docItem.jsDoc.params) {
                const name = tag.name.getText(docItem.sf);
                let comment = tag.comment;
                if (typeof comment === 'string') {
                    if ((comment === null || comment === void 0 ? void 0 : comment.indexOf('- ')) === 0)
                        // remove leading '- ' if present
                        comment = comment.slice(2);
                    md += `+ **${name}**\n  + ${comment}\n`;
                }
            }
            md += '\n';
        }
        return md;
    }
    returnsDetails(docItem) {
        let md = '';
        if (docItem.jsDoc.returns.length > 0) {
            md += `Returns\n\n`;
            for (const t of docItem.jsDoc.returns) {
                md += `${t.comment}\n\n`;
            }
        }
        return md;
    }
    throwsDetails(docItem) {
        let md = '';
        if (docItem.jsDoc.throws.length > 0) {
            md += `Throws\n\n`;
            for (const tag of docItem.jsDoc.throws) {
                md += `${tag.comment}\n\n`;
            }
        }
        return md;
    }
    examplesDetails(docItem) {
        let md = '';
        if (docItem.jsDoc.examples.length > 0) {
            md += `Example${docItem.jsDoc.examples.length > 1 ? 's' : ''}\n\n`;
            for (const e of docItem.jsDoc.examples) {
                md += `${e}\n`;
            }
        }
        return md;
    }
    commentsDetails(docItem) {
        let md = '';
        for (const comment of docItem.jsDoc.comments) {
            md += `${comment}\n\n`;
        }
        return md;
    }
}
class DocVariable extends DocBase {
    constructor(sup) { super(sup, 'Variable', 'Variables'); }
    getName(item, sf) {
        return item.name.getText(sf);
    }
    filterItem(item) {
        const items = [];
        if (typescript_1.default.isVariableStatement(item) && item.modifiers &&
            (this.sup.nothingPrivate || item.modifiers.some(m => m.kind === typescript_1.default.SyntaxKind.ExportKeyword))) {
            for (const v of item.declarationList.declarations) {
                items.push(v);
            }
        }
        return items;
    }
}
class DocType extends DocBase {
    constructor(sup) { super(sup, 'Type', 'Types'); }
    getName(item) {
        return item.name.text;
    }
    filterItem(item) {
        if (typescript_1.default.isTypeAliasDeclaration(item) &&
            (this.sup.nothingPrivate || this.isExportedDeclaration(item)))
            return [item];
        return [];
    }
}
class DocFunction extends DocBase {
    constructor(sup) { super(sup, 'Function', 'Functions', 'Details'); }
    getName(item) {
        var _a;
        return ((_a = item.name) === null || _a === void 0 ? void 0 : _a.text) || '';
    }
    filterItem(item) {
        if (typescript_1.default.isFunctionDeclaration(item) &&
            (this.sup.nothingPrivate || this.isExportedDeclaration(item)))
            return [item];
        return [];
    }
    toMarkDownTs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        const printer = this.sup.printer;
        let mdts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n, sf);
        if (n.body && !docItem.jsDoc.publicBody) {
            // Remove the function body
            const bodyts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n.body, sf);
            mdts = mdts.slice(0, mdts.length - bodyts.length);
        }
        return mdts;
    }
    toMarkDownDetails(docItem) {
        let md = '';
        md += this.returnsDetails(docItem);
        md += this.argumentsDetails(docItem);
        md += this.throwsDetails(docItem);
        return md;
    }
}
class DocProperty extends DocBase {
    constructor(sup) { super(sup, 'Property', 'Properties'); }
    getName(item) {
        if (typescript_1.default.isIdentifier(item.name))
            return item.name.text;
        return '';
    }
    filterItem(item) {
        if (typescript_1.default.isPropertyDeclaration(item) && this.isNotPrivate(item))
            return [item];
        return [];
    }
    toMarkDownDetails(docItem) {
        let md = '';
        md += this.examplesDetails(docItem);
        const comments = this.commentsDetails(docItem);
        if (md || comments) {
            const mdts = '```ts\n' + this.toMarkDownTs(docItem) + '\n```\n';
            let intro = `${this.sup.headingLevelMd(4)} Property ${docItem.name}\n\n`;
            if (comments)
                intro += comments;
            md = `${intro}${mdts}\n${md}`;
        }
        return md;
    }
}
class DocConstructor extends DocBase {
    constructor(sup) { super(sup, 'Constructor', 'Constructors', 'Arguments'); }
    getName(item) {
        return 'constructor';
    }
    filterItem(item) {
        if (typescript_1.default.isConstructorDeclaration(item) && this.isNotPrivate(item))
            return [item];
        return [];
    }
    toMarkDownTs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        const printer = this.sup.printer;
        let mdts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n, sf);
        if (n['body'] && !(this.sup.nothingPrivate || docItem.jsDoc.publicBody)) {
            // Remove the body from documentation typescript
            const bodyts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n['body'], sf);
            mdts = this.removeTs(mdts, bodyts);
        }
        return mdts;
    }
    toMarkDownDetails(docItem) {
        let md = '';
        md += this.argumentsDetails(docItem);
        md += this.throwsDetails(docItem);
        md += this.examplesDetails(docItem);
        const comments = this.commentsDetails(docItem);
        if (md || comments) {
            const mdts = '```ts\n' + this.toMarkDownTs(docItem) + '\n```\n';
            let intro = `${this.sup.headingLevelMd(4)} Constructor\n\n`;
            if (comments)
                intro += comments;
            md = `${intro}${mdts}\n${md}`;
        }
        return md;
    }
}
class DocMethod extends DocBase {
    constructor(sup) { super(sup, 'Method', 'Methods'); }
    getName(item) {
        if (typescript_1.default.isIdentifier(item.name))
            return item.name.text;
        return '';
    }
    filterItem(item) {
        if (typescript_1.default.isMethodDeclaration(item) && this.isNotPrivate(item))
            return [item];
        return [];
    }
    toMarkDownTs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        const printer = this.sup.printer;
        let mdts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n, sf);
        if (n['body'] && !(this.sup.nothingPrivate || docItem.jsDoc.publicBody)) {
            // Remove the body from documentation typescript
            const bodyts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n['body'], sf);
            mdts = this.removeTs(mdts, bodyts);
        }
        return mdts;
    }
    toMarkDownDetails(docItem) {
        let md = '';
        md += this.returnsDetails(docItem);
        md += this.argumentsDetails(docItem);
        md += this.throwsDetails(docItem);
        md += this.examplesDetails(docItem);
        const comments = this.commentsDetails(docItem);
        if (md || comments) {
            const mdts = '```ts\n' + this.toMarkDownTs(docItem) + '\n```\n';
            let intro = `${this.sup.headingLevelMd(4)} Method ${docItem.name}\n\n`;
            if (comments)
                intro += comments;
            md = `${intro}${mdts}\n${md}`;
        }
        return md;
    }
}
class DocClass extends DocBase {
    constructor(sup) { super(sup, 'Class', 'Classes'); }
    getName(item) {
        var _a;
        return ((_a = item.name) === null || _a === void 0 ? void 0 : _a.text) || '';
    }
    filterItem(item) {
        if (typescript_1.default.isClassDeclaration(item) &&
            (this.sup.nothingPrivate || this.isExportedDeclaration(item)))
            return [item];
        return [];
    }
    extractMemberDocs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        let docs = [
            new DocConstructor(this.sup),
            new DocProperty(this.sup),
            new DocMethod(this.sup),
        ];
        for (const ce of n.members) {
            for (const doc of docs) {
                doc.tryAddItem(ce, sf, docItem);
            }
        }
        // Eliminate empty doc categories
        docs = docs.filter(d => d.docItems.length > 0);
        return docs;
    }
    toMarkDownTs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        const printer = this.sup.printer;
        // class definition typescript including all members and function bodies...
        let mdts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n, sf);
        for (const ce of n.members) {
            const r = getJsDocInfo(ce, docItem.name);
            let isPrivate = false;
            if (!this.sup.nothingPrivate &&
                ((ce['modifiers'] && ce['modifiers'].some(m => m.kind === typescript_1.default.SyntaxKind.PrivateKeyword)) ||
                    r.tags.some(t => typescript_1.default.isJSDocPrivateTag(t)))) {
                isPrivate = true;
                // Remove entire member
                const memberts = printer.printNode(typescript_1.default.EmitHint.Unspecified, ce, sf);
                mdts = this.removeTs(mdts, memberts, true);
            }
            else if (ce['body'] && !r.publicBody) {
                // Remove the body from documentation typescript
                const bodyts = printer.printNode(typescript_1.default.EmitHint.Unspecified, ce['body'], sf);
                mdts = this.removeTs(mdts, bodyts);
            }
            else if (ce['initializer'] && r.privateInitializer) {
                const initializerts = printer.printNode(typescript_1.default.EmitHint.Unspecified, ce['initializer'], sf);
                mdts = this.removeTs(mdts, `= ${initializerts};`);
            }
        }
        return mdts;
    }
    toMarkDownDetails(docItem) {
        let md = '';
        for (const doc of docItem.memberDocs) {
            for (const item of doc.docItems) {
                const details = doc.toMarkDownDetails(item);
                if (details)
                    md += details;
            }
        }
        return md;
    }
}
class DocPropertySignature extends DocBase {
    constructor(sup) { super(sup, 'Property', 'Properties'); }
    getName(item) {
        if (typescript_1.default.isIdentifier(item.name))
            return item.name.text;
        return '';
    }
    filterItem(item) {
        if (typescript_1.default.isPropertySignature(item) && this.isNotPrivate(item))
            return [item];
        return [];
    }
    toMarkDownDetails(docItem) {
        let md = '';
        md += this.examplesDetails(docItem);
        const comments = this.commentsDetails(docItem);
        if (md || comments) {
            const mdts = '```ts\n' + this.toMarkDownTs(docItem) + '\n```\n';
            let intro = `${this.sup.headingLevelMd(4)} Property ${docItem.name}\n\n`;
            if (comments)
                intro += comments;
            md = `${intro}${mdts}\n${md}`;
        }
        return md;
    }
}
class DocMethodSignature extends DocBase {
    constructor(sup) { super(sup, 'Method', 'Methods'); }
    getName(item) {
        if (typescript_1.default.isIdentifier(item.name))
            return item.name.text;
        return '';
    }
    filterItem(item) {
        if (typescript_1.default.isMethodSignature(item) && this.isNotPrivate(item))
            return [item];
        return [];
    }
    toMarkDownTs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        const printer = this.sup.printer;
        let mdts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n, sf);
        if (n['body'] && !(this.sup.nothingPrivate || docItem.jsDoc.publicBody)) {
            // Remove the body from documentation typescript
            const bodyts = printer.printNode(typescript_1.default.EmitHint.Unspecified, n['body'], sf);
            mdts = this.removeTs(mdts, bodyts);
        }
        return mdts;
    }
    toMarkDownDetails(docItem) {
        let md = '';
        md += this.returnsDetails(docItem);
        md += this.argumentsDetails(docItem);
        md += this.throwsDetails(docItem);
        md += this.examplesDetails(docItem);
        const comments = this.commentsDetails(docItem);
        if (md || comments) {
            const mdts = '```ts\n' + this.toMarkDownTs(docItem) + '\n```\n';
            let intro = `${this.sup.headingLevelMd(4)} Method ${docItem.name}\n\n`;
            if (comments)
                intro += comments;
            md = `${intro}${mdts}\n${md}`;
        }
        return md;
    }
}
class DocInterface extends DocBase {
    constructor(sup) { super(sup, 'Interface', 'Interfaces'); }
    getName(item) {
        return item.name.text;
    }
    filterItem(item) {
        if (typescript_1.default.isInterfaceDeclaration(item) &&
            (this.sup.nothingPrivate || this.isExportedDeclaration(item)))
            return [item];
        return [];
    }
    extractMemberDocs(docItem) {
        const n = docItem.item;
        const sf = docItem.sf;
        let docs = [
            new DocPropertySignature(this.sup),
            new DocMethodSignature(this.sup),
        ];
        for (const ce of n.members) {
            for (const doc of docs) {
                doc.tryAddItem(ce, sf, docItem);
            }
        }
        // Eliminate empty doc categories
        docs = docs.filter(d => d.docItems.length > 0);
        return docs;
    }
    toMarkDownDetails(docItem) {
        let md = '';
        for (const doc of docItem.memberDocs) {
            for (const item of doc.docItems) {
                const details = doc.toMarkDownDetails(item);
                if (details)
                    md += details;
            }
        }
        return md;
    }
}
/**
 * Uses the Typescript compiler to parse source tree given a top level source file such as `index.ts`.
 *
 * Extract the exported API interfaces, classes, types, functions and variables.
 *
 * Generate GitHub friendly MarkDown documentation for the extracted API leveraging TypeScript type information
 * and merging JSDoc style documentation comments.
 *
 * The following JSDoc tags are supported:
 *
 *    `@publicbody` Applied to a class method or function. Adds the function body to the documentation with embedded comments removed.
 *
 *    `@private` Applied to an exported or publicly accessible member keeps it out of documentation.
 *
 *    `@privateinitializer` Applied to a class property with an initializer will prevent the initializer from appearing in the documentation.
 */
class Ts2Md {
    /**
     * Construct a new instance configured for `run` method to be called next.
     *
     * @param options Must be provided. inputFilename defaults to `./src/index.ts`
     */
    constructor(options) {
        this.options = options;
        this.sourceFiles = [];
        this.docs = [];
        options.inputFilename || (options.inputFilename = './src/index.ts');
        this.nothingPrivate = options.nothingPrivate || false;
        this.filePath = path.resolve(options.inputFilename);
        this.fileName = path.parse(this.filePath).name;
        const compOpts = {
            allowJs: false,
            target: typescript_1.default.ScriptTarget.Latest
        };
        const progOpts = {
            rootNames: [this.filePath],
            options: compOpts
        };
        this.program = typescript_1.default.createProgram(progOpts);
        const printOpts = {
            newLine: typescript_1.default.NewLineKind.LineFeed,
            removeComments: true,
            omitTrailingSemicolon: true
        };
        const printHandlers = {};
        this.printer = typescript_1.default.createPrinter(printOpts, printHandlers);
    }
    /**
     * Generates the documentation markdown and write's it to output file
     * and/or merges it to README.md
     */
    run() {
        this.sourceFiles = this.parseSourceFiles(this.program);
        this.docs = this.extractDocs(this.sourceFiles);
        this.markDown = this.generateMarkDown(this.docs);
        if (this.markDown) {
            if (this.options.outputFilename)
                this.outputPath = this.writeMarkDownToOuput(this.markDown, this.options.outputFilename);
            if (this.options.readmeMerge)
                this.readmeMerge(this.markDown);
        }
    }
    /**
     * @private
     */
    headingLevelMd(relativeLevel) {
        relativeLevel += this.options.firstHeadingLevel - 1;
        if (this.options.noTitle)
            relativeLevel--;
        return '#'.repeat(relativeLevel + this.options.firstHeadingLevel - 1);
    }
    parseSourceFiles(program) {
        return program.getSourceFiles().filter(sf => sf.fileName.indexOf('node_modules') === -1);
    }
    extractDocs(sourceFiles) {
        let docs = [
            new DocInterface(this),
            new DocClass(this),
            new DocFunction(this),
            new DocType(this),
            new DocVariable(this)
        ];
        for (const sf of sourceFiles) {
            console.log(`SourceFile ${sf.fileName}`);
            typescript_1.default.forEachChild(sf, node => {
                if (typescript_1.default.isStatement(node)) {
                    for (const doc of docs) {
                        doc.tryAddItem(node, sf);
                    }
                }
            });
        }
        // Eliminate empty doc categories
        docs = docs.filter(d => d.docItems.length > 0);
        return docs;
    }
    generateMarkDown(docs) {
        let md = '';
        if (!this.options.noTitle) {
            md += `${this.headingLevelMd(1)} API\n\n`;
        }
        else {
            md += '\n';
        }
        let linksMd = `Links: [API](#api)`;
        for (const doc of docs) {
            linksMd += `, [${doc.labelPlural}](#${doc.labelPlural.toLowerCase()})`;
        }
        linksMd += '\n\n';
        md += linksMd;
        for (const doc of this.docs) {
            md += `${this.headingLevelMd(2)} ${doc.labelPlural}\n\n`;
            if (doc.docItems.length > 1) {
                md += this.generateDocItemLinksTable(doc);
                md += `\n${linksMd}---\n\n`;
            }
            for (const docItem of doc.docItems) {
                md += doc.toMarkDown(docItem);
                md += linksMd;
                md += '---\n';
            }
        }
        return md;
    }
    /**
     * Generates a markdown table of named links to docItem documentation headers.
     * @param doc The category of documentation for which to generate the table.
     * @returns markdown table of named links to docItem documentation headers
     */
    generateDocItemLinksTable(doc) {
        let md = '';
        const nameLinks = doc.docItems.map(d => ({ name: d.name, linkMd: doc.toMarkDownRefLink(d) }));
        nameLinks.sort((a, b) => a.name < b.name ? -1 : a.name === b.name ? 0 : 1);
        let cols = 1, rows = nameLinks.length;
        while (cols < 3 && rows > 12) {
            cols++;
            rows = Math.ceil(nameLinks.length / cols);
        }
        for (let col = 1; col <= cols; col++) {
            md += '| ';
        }
        md += '|\n';
        for (let col = 1; col <= cols; col++) {
            md += '| --- ';
        }
        md += '|\n';
        for (let row = 1; row <= rows; row++) {
            for (let col = 1; col <= cols; col++) {
                const i = (col - 1) * rows + row - 1;
                const d = i < nameLinks.length ? nameLinks[i].linkMd : '';
                if (col === 1)
                    md += '|';
                md += ` ${d} |`;
            }
            md += '\n';
        }
        return md;
    }
    writeMarkDownToOuput(markDown, outputFilename) {
        const outputPath = path.resolve(outputFilename);
        if (this.options.outputReplace) {
            try {
                fs.unlinkSync(outputPath);
            }
            catch ( /* */_a) { /* */ }
        }
        fs.writeFileSync(outputPath, markDown);
        return outputPath;
    }
    readmeMerge(markDown) {
        (0, mdMerge_1.mdMerge)(markDown);
    }
}
exports.Ts2Md = Ts2Md;
/**
 * Generate Typescript documentation and merge into README.md
 *
 * Attempts to validate options, constructs an instance of Ts2Md with those options, and runs the generation method.
 *
 * 1. Function argument is used if provided.
 *
 * 2. Looks for `./ts2md.json`
 *
 * 3. Default options.
 *
 * Default options are:
 *
 * ```json
 * {
 *   "inputFilename": "./src/index.ts",
 *   "outputFilename": "./apiDoc.md",
 *   "firstHeadingLevel": 2,
 *   "noTitle": true,
 *   "outputReplace": true,
 *   "readmeMerge": true
 * }
 * ```
 * @param options Optional options to control markdown generation.
 * @publicbody
 */
function ts2md(options) {
    if (!options) {
        try {
            const configPath = path.resolve('./ts2md.json');
            const json = fs.readFileSync(configPath, { encoding: 'utf8' });
            options = JSON.parse(json);
        }
        catch ( /* */_a) { /* */ }
    }
    options || (options = {
        inputFilename: './src/index.ts',
        outputFilename: '',
        firstHeadingLevel: 2,
        noTitle: true,
        outputReplace: true,
        readmeMerge: true,
        //nothingPrivate: true
    });
    new Ts2Md(options).run();
}
exports.ts2md = ts2md;
console.log('running ts2md()');
ts2md();
//# sourceMappingURL=ts2md.js.map